@@ -1,36 +1,42 @@
-##### panel generating graphic functions for various tree models
-
 ## profile plot visualization function
-node_profileplot <- function(mobobj, what = c("items", "thresholds", "discriminations"),
-  parg = list(type = NULL, ref = NULL, alias = TRUE), id = TRUE, names = FALSE,
-  abbreviate = TRUE, index = TRUE, ref = TRUE, col = "black", border = col,
-  linecol = "black", refcol = "lightgray", bg = "white", cex = 0.5, pch = 21,
-  xscale = NULL, yscale = NULL, ylines = 2, ...)
+node_profileplot <- function(mobobj, what = c("item", "coef", "threshold", "discrimination", "guessing", "upper"),
+                             parg = list(type = NULL, ref = NULL, alias = TRUE, logit = FALSE), id = TRUE, names = FALSE,
+                             abbreviate = TRUE, index = TRUE, ref = TRUE, col = "black", border = col,
+                             linecol = "black", refcol = "lightgray", bg = "white", cex = 0.5, pch = 21,
+                             xscale = NULL, yscale = NULL, ylines = 2, ...)
 {
   ## check input
   what <- match.arg(what)
-  if (what == "thresholds") type <- parg$type
+  if (what == "threshold") type <- parg$type
   refpar <- parg$ref
   alias <- if (is.null(parg$alias)) TRUE else parg$alias
   addargs <- list(...)
   if ("worth" %in% names(addargs)) warning("the argument 'worth' is deprecated and not longer used")
+  logit <- parg$logit
 
   ## node ids
-  node <- nodeids(mobobj, terminal = FALSE)
-  
-  ## get all coefficients 
-  if (what == "items") {
-    cf <- apply_to_models(mobobj, node, FUN = function(z) coef(itempar(z, ref = refpar, alias = alias, vcov = FALSE)))
-  } else if (what == "thresholds") {
+  node <- partykit::nodeids(mobobj, terminal = FALSE)
+
+  ## get all coefficients
+  if (what == "item") {
+    cf <- apply_to_models(mobobj, node, FUN = function(z) itempar(z, ref = refpar, alias = alias, vcov = FALSE))
+  } else if (what == "coef") {
+    cf <- apply_to_models(mobobj, node, FUN = function(z) coef(z))
+  } else if (what == "threshold") {
     cf <- apply_to_models(mobobj, node, FUN = function(z) coef(threshpar(z, type = type, ref = refpar, alias = alias, vcov = FALSE), type = "matrix"))
-  } else {
+  } else if (what == "discrimination"){
     cf <- apply_to_models(mobobj, node, FUN = function(z) coef(discrpar(z, ref = refpar, alias = alias, vcov = FALSE)))
+  } else if (what == "guessing"){
+    cf <- apply_to_models(mobobj, node, FUN = function(z) coef(guesspar(z, alias = alias, logit = logit, vcov = FALSE)))
+  } else if (what == "upper"){
+    cf <- apply_to_models(mobobj, node, FUN = function(z) coef(upperpar(z, alias = alias, logit = logit, vcov = FALSE)))
   }
+
   names(cf) <- node
 
   ## labeling
   if (isTRUE(names)) {
-    nms <- if (what != "thresholds") lapply(cf, names) else lapply(cf, rownames)
+    nms <- if (what != "threshold") lapply(cf, names) else lapply(cf, rownames)
   } else if (is.character(names)) {
     nms <- split(rep(names, length(node)), f = rep(1:length(node), each = length(names)))
   } else {
@@ -40,12 +46,13 @@ node_profileplot <- function(mobobj, what = c("items", "thresholds", "discrimina
       lab[c(1, m)] <- c(1, m)
       pr <- pretty(1:m, n = 4)
       pr <- pr[pr > 1 & pr < m]
-      lab[pr] <- pr    
+      lab[pr] <- pr
       lab
     })
     abbreviate <- FALSE
   }
-  
+
+
   ## abbreviation
   if (is.logical(abbreviate)) {
     nlab <- max(unlist(lapply(nms, function (j) nchar(j))))
@@ -53,9 +60,10 @@ node_profileplot <- function(mobobj, what = c("items", "thresholds", "discrimina
   }
   nms <- lapply(nms, function (j) abbreviate(j, abbreviate))
 
+
   ## axis scale
   if (index) {
-    x <- if (what == "thresholds") 1:nrow(cf[[1]]) else 1:length(cf[[1]])
+    x <- if (what == "threshold") 1:nrow(cf[[1]]) else 1:length(cf[[1]])
     if (is.null(xscale)) xscale <- range(x) + c(-0.1, 0.1) * diff(range(x))
   } else {
     if (what == "thresholds") {
@@ -76,7 +84,7 @@ node_profileplot <- function(mobobj, what = c("items", "thresholds", "discrimina
 
     ## node index
     idn <- id_node(node)
-    
+
     ## get cfs and labels
     cfi <- cf[[idn]]
     if(any(!is.finite(cfi))) {
@@ -87,8 +95,8 @@ node_profileplot <- function(mobobj, what = c("items", "thresholds", "discrimina
 
     ## viewport setup
     top_vp <- viewport(layout = grid.layout(nrow = 2, ncol = 3,
-                           widths = unit(c(ylines, 1, 1), c("lines", "null", "lines")),  
-                           heights = unit(c(1, 1), c("lines", "null"))),
+                                            widths = unit(c(ylines, 1, 1), c("lines", "null", "lines")),
+                                            heights = unit(c(1, 1), c("lines", "null"))),
                        width = unit(1, "npc"), height = unit(1, "npc") - unit(2, "lines"),
                        name = paste("node_profileplot", idn, sep = ""))
     pushViewport(top_vp)
@@ -102,9 +110,9 @@ node_profileplot <- function(mobobj, what = c("items", "thresholds", "discrimina
     grid.text(mainlab)
     popViewport()
 
-    ## actual plot  
-    plot_vpi <- viewport(layout.pos.col = 2, layout.pos.row = 2, xscale = xscale, yscale = yscale, 
-                               name = paste("node_profileplot", idn, "plot", sep = ""))
+    ## actual plot
+    plot_vpi <- viewport(layout.pos.col = 2, layout.pos.row = 2, xscale = xscale, yscale = yscale,
+                         name = paste("node_profileplot", idn, "plot", sep = ""))
     pushViewport(plot_vpi)
     grid.lines(xscale, c(mean(cfi), mean(cfi)), gp = gpar(col = refcol), default.units = "native")
     if(index) {
@@ -118,8 +126,8 @@ node_profileplot <- function(mobobj, what = c("items", "thresholds", "discrimina
         grid.points(x, cfi, gp = gpar(col = border, fill = col, cex = cex), pch = pch, default.units = "native")
       }
       grid.xaxis(at = x, label = nmsi)
-    } else {	
-      if (what == "thresholds") {
+    } else {
+      if (what == "threshold") {
         for (j in 1:ncol(cfi)) grid.text(paste0(nmsi, "-C", j), x[j], y = cfi[, j], default.units = "native")
       } else {
         grid.text(nmsi, x = x, y = cfi, default.units = "native")
@@ -130,16 +138,14 @@ node_profileplot <- function(mobobj, what = c("items", "thresholds", "discrimina
 
     upViewport(2)
   }
-  
+
   return(panelfun)
 }
 class(node_profileplot) <- "grapcon_generator"
 
-
-## region plot visualization function
 node_regionplot <- function(mobobj, names = FALSE, abbreviate = TRUE, type = c("mode", "median", "mean"),
-  ref = NULL, ylim = NULL, off = 0.1, col_fun = gray.colors, bg = "white",
-  uo_show = TRUE, uo_col = "red", uo_lty = 2, uo_lwd = 1.25, ylines = 2)    
+                            ref = NULL, ylim = NULL, off = 0.1, col_fun = gray.colors, bg = "white",
+                            uo_show = TRUE, uo_col = "red", uo_lty = 2, uo_lwd = 1.25, ylines = 2)
 {
   ## check input
   stopifnot(!is.null(mobobj))
@@ -173,12 +179,12 @@ node_regionplot <- function(mobobj, names = FALSE, abbreviate = TRUE, type = c("
       lab[c(1, m)] <- c(1, m)
       pr <- pretty(1:m, n = 4)
       pr <- pr[pr > 1 & pr < m]
-      lab[pr] <- pr    
+      lab[pr] <- pr
       lab
     })
     abbreviate <- FALSE
   }
-  
+
   ## abbreviation
   if (is.logical(abbreviate)) {
     nlab <- max(unlist(lapply(names, function (j) nchar(j))))
@@ -196,7 +202,7 @@ node_regionplot <- function(mobobj, names = FALSE, abbreviate = TRUE, type = c("
     delta_unsorted <- delta_lst[[id]]
     lab <- paste("node", id, sep = "")
     namesi <- names[[id]]
-    
+
     ## compute sorted absolute item threshold parameters (code borrowed from psychotools::regionplot())
     delta_sorted <- delta_unsorted
     us <- sapply(delta_unsorted, is.unsorted)
@@ -205,7 +211,7 @@ node_regionplot <- function(mobobj, names = FALSE, abbreviate = TRUE, type = c("
       for (j in usj) {
         tpj <- delta_unsorted[[j]]
         nj <- length(tpj)
-        
+
         ## check if there is a point with a biggest parameter, if yes, take mean
         for (i in 1:nj) {
           if (all(tpj[i] > tpj[(i+1):nj])) {
@@ -214,21 +220,21 @@ node_regionplot <- function(mobobj, names = FALSE, abbreviate = TRUE, type = c("
             break
           }
         }
-        
+
         ## recursive sorting if there is still unorder (e.g. 4, 2, 3, 1)
         while(is.unsorted(tpj)) {
           uo_pos <- which(diff(tpj) < 0)		     # locate unordered parameters, returns position of the first
           tpj[uo_pos] <- (tpj[uo_pos] + tpj[uo_pos + 1]) / 2 # replace first with location of intersection of ccc curves (= (eps1 + eps2)/ 2)
           tpj <- tpj[-(uo_pos + 1)]			     # remove second
         }
-        
+
         delta_sorted[[j]] <- tpj
       }
     }
 
     ## terminal panel viewport setup
     top.vp <- viewport(layout = grid.layout(nrow = 2, ncol = 1, widths = unit(1, "null"), heights = unit(c(1, 1), c("lines", "null"))),
-        		     width = unit(1, "npc"), height = unit(1, "npc") - unit(2, "lines"), name = paste(lab, "_effects", sep = ""))
+                       width = unit(1, "npc"), height = unit(1, "npc") - unit(2, "lines"), name = paste(lab, "_effects", sep = ""))
     pushViewport(top.vp)
     grid.rect(gp = gpar(fill = bg, col = 0), name = paste(lab, "_border", sep = ""))
 
@@ -236,7 +242,7 @@ node_regionplot <- function(mobobj, names = FALSE, abbreviate = TRUE, type = c("
     pushViewport(viewport(layout.pos.col = 1, layout.pos.row = 1, name = paste(lab, "_title_vp", sep = "")))
     grid.text(paste("Node ", id, " (n = ", info_node(node)$nobs, ")", sep = ""), name = paste(lab, "_title", sep = ""))
     upViewport()
-    
+
     ## finally the actual plot
     pushViewport(viewport(layout.pos.col = 1, layout.pos.row = 2, name = lab))
     lab <- paste(lab, "_plot", sep = "")
@@ -255,10 +261,10 @@ node_regionplot <- function(mobobj, names = FALSE, abbreviate = TRUE, type = c("
 
     ## plot rectangles per item
     for (j in seq_along(delta_sorted)) {
-        ncat <- length(delta_sorted[[j]]) + 1
-        grid.rect(x = rep.int(xi[j], ncat), y = c(ylim[1], delta_sorted[[j]]), width = rep.int(1, ncat),
-        		height = diff.default(c(ylim[1], delta_sorted[[j]], ylim[2])), just = c("left", "bottom"),
-        		gp = gpar(fill = col_fun(ncat)), default.units = "native", name = paste(lab, "_item", j, "_rect", sep = ""))
+      ncat <- length(delta_sorted[[j]]) + 1
+      grid.rect(x = rep.int(xi[j], ncat), y = c(ylim[1], delta_sorted[[j]]), width = rep.int(1, ncat),
+                height = diff.default(c(ylim[1], delta_sorted[[j]], ylim[2])), just = c("left", "bottom"),
+                gp = gpar(fill = col_fun(ncat)), default.units = "native", name = paste(lab, "_item", j, "_rect", sep = ""))
     }
 
     ## if requested: indicate unordered parameters
@@ -267,7 +273,7 @@ node_regionplot <- function(mobobj, names = FALSE, abbreviate = TRUE, type = c("
       for (j in uo_items) {
         uo_pars <- setdiff(delta_unsorted[[j]], delta_sorted[[j]])
         grid.polyline(x = rep(c(xi[j], xi[j] + 1), length(uo_pars)), y = rep(uo_pars, each = 2), default.units = "native", id = rep(1:length(uo_pars), each = 2),
-        	      name = paste(lab, "item", j, "_uolines", sep = ""), gp = gpar(col = uo_col, lwd = uo_lwd, lty = uo_lty))
+                      name = paste(lab, "item", j, "_uolines", sep = ""), gp = gpar(col = uo_col, lwd = uo_lwd, lty = uo_lty))
       }
     }
 
@@ -276,11 +282,11 @@ node_regionplot <- function(mobobj, names = FALSE, abbreviate = TRUE, type = c("
     grid.xaxis(at = (xi[-(m+1)] + 0.5), label = namesi, main = TRUE, name = paste(lab, "_xaxis-bottom", sep = ""))
     grid.yaxis(main = TRUE, name = paste(lab, "_yaxis-left", sep = ""))
     upViewport()
-    
+
     ## add descriptions
     pushViewport(lmargin.vp)
     upViewport(2)
-    
+
     ## go back to uper vp
     upViewport(2)
   }
@@ -511,4 +517,3 @@ node_mptplot <- function(mobobj, id = TRUE,
     return(rval)
 }
 class(node_mptplot) <- "grapcon_generator"
-
